@@ -21,11 +21,13 @@ typedef enum {
 // clang-format on
 
 GtkWidget *smtk_keys_area_new(SmtkKeyMode mode, SmtkKeyAlignment alignment,
-			      bool draw_border, int timeout);
+			      bool draw_border, double margin_ratio,
+			      int timeout);
 void smtk_keys_area_set_mode(SmtkKeysArea *area, SmtkKeyMode mode);
 void smtk_keys_area_set_alignment(SmtkKeysArea *area,
 				  SmtkKeyAlignment alignment);
 void smtk_keys_area_set_draw_border(SmtkKeysArea *area, bool draw_border);
+void smtk_keys_area_set_margin_ratio(SmtkKeysArea *area, double margin_ratio);
 void smtk_keys_area_set_timeout(SmtkKeysArea *area, int timeout);
 void smtk_keys_area_add_key(SmtkKeysArea *area, char *key);
 
